<?php
if(!defined('PLX_ROOT')) { exit; }

/* A faire :
 * Si le plugin est supprimé alors supprimer le fichier de données s'il existe
 * */

/**
 * kzEvents addon for PluXml
 *
 * Gestion évènementielle. Ajoute un champ date aux articles dans un fichier indépendant comme tags.xml.
 * Les dates de publication sont stockées dans le tableau $_eVents du plugin
 * Les dates sont sauvegardées au format "2024-09-22"
 * Incorporer le fichier sample.php au contenu d'une page statique
 *
 * Exemple de code PHP à insérer pour l'affichage d'un article :
 *
 * <?php
 * if(class_exists('kzEvents')) {
 * 		$dateEvent = $plxShow->plxMotor->plxRecord_arts->f(kzEvents::FIELDNAME);
 * 		if(!empty($dateEvent)) {
 * ?>
 * 		<p>
 * 			<span><?= $plxMotor->plxPlugins->aPlugins['kzEvents']->getLang('ARTICLE_EVENT') ?></span>
 * 			<time datetime="<?= $dateEvent ?>">
 * 				<?= plxDate::formatDate(str_replace('-', '', $dateEvent) , '#num_day #month #num_year(4)'); ?>
 * 			</time>
 * 		</p>
 * <?php
 * 		}
 * }
 * ?>
 *
 * Compatible PHP 8.3.11 et PluXml 5.8.16 et PluXml 5.9.0-rc7
 *
 * @author Jean-Pierre Pourrez @bazooka07
 * @version 1.1.1
 * @update 2024-09-25
 * @date 2024-09-19
 * */
class kzEvents extends plxPlugin {
	const FIELDNAME = 'kzEvent';
	const FIELDTYPE = 'date';
	const DATAFILE = PLX_ROOT . PLX_CONFIG_PATH . 'plugins/' . __CLASS__.'-data.xml';
	const XML_HEADER = "<?xml version='1.0' encoding='" . PLX_CHARSET ."'?>" . PHP_EOL;
	const HOOKS = array(
		'plxMotorConstruct',
		'plxMotorParseArticle',
	);
	const ADMIN_HOOKS = array(
		'AdminArticlePreview',
		'AdminArticlePostData',
		'AdminArticleParseData',
		'AdminArticleInitData',
		'plxAdminEditArticleEnd',
		'plxAdminDelArticle',
		'AdminArticleContent',
		'AdminArticleSidebar',
		'AdminIndexFoot',
	);
	const LANG = array(
		'de' => array(
			'ARTICLE_EVENT' => 'Veranstaltungsdatum',
			'NO_EVENT' => 'Keine Veranstaltungen',
		),
		'en' => array(
			'ARTICLE_EVENT' => 'Date of the event',
			'NO_EVENT' => 'No events',
		),
		'es' => array(
			'ARTICLE_EVENT' => 'Fecha del evento',
			'NO_EVENT' => 'Sin eventos',
		),
		'fr' => array(
			'ARTICLE_EVENT' => 'Date de l\'évènement',
			'NO_EVENT' => 'Aucun évènement',
		),
		'it' => array(
			'ARTICLE_EVENT' => 'Data dell\'evento',
			'NO_EVENT' => 'Nessun evento',
		),
		'nl' => array(
			'ARTICLE_EVENT' => 'Datum evenement',
			'NO_EVENT' => 'Geen evenementen',
		),
		'oc' => array(
			'ARTICLE_EVENT' => 'Data de l\'eveniment',
			'NO_EVENT' => 'Pas d\'eveniments',
		),
		'pl' => array(
			'ARTICLE_EVENT' => 'Data wydarzenia',
			'NO_EVENT' => 'Brak wydarzeń',
		),
		'pt' => array(
			'ARTICLE_EVENT' => 'Data do evento',
			'NO_EVENT' => 'Nenhum evento',
		),
		'ro' => array(
			'ARTICLE_EVENT' => 'Data evenimentului',
			'NO_EVENT' => 'Fără evenimente',
		),
		'ru' => array(
			'ARTICLE_EVENT' => 'Дата мероприятия',
			'NO_EVENT' => 'Нет событий',
		),
	);
	const BEGIN_CODE = '<?php # ' . __CLASS__ . ' plugin' . PHP_EOL;
	const END_CODE = PHP_EOL . '?>';

	private $_aEvents;

	public function __construct($lang) {
		parent::__construct($lang);

		foreach(self::HOOKS as $hook) {
			parent::addHook($hook, $hook);
		}

		if(defined('PLX_ADMIN')) {
			foreach(self::ADMIN_HOOKS as $hook) {
				parent::addHook($hook, $hook);
			}
		} else {
			# Hook à appeler depuis le site
			parent::addHook('kzEvent', 'kzEvent');
		}

		# Mimics plxPlugins::getPluginLang()
		$this->default_lang = $lang;
		$this->aLang = self::LANG[$lang];
	}

	/*
	 * Enregistrement de la valeur du nouveau champ pour l'article en cours d'édition
	 * */
	public function editEvent($artId, $data) {
		if(is_array($data) and !empty($data[self::FIELDNAME])) {

			if(array_key_exists($artId, $this->_aEvents) and empty(array_diff_assoc($data, $this->_aEvents[$artId]))) {
				# Aucun changement
				return;
			}

			# Mise à jour
			$this->_aEvents[$artId] = $data;
		} elseif(!array_key_exists($artId, $this->_aEvents)) {
			# Aucun changement
			return;
		} else {
			unset($this->_aEvents[$artId]);
		}
		# sauvegarde
		$this->editEvents();
	}

	/*
	 * Enregistrement du nouveau champ de tous les articles dans un fichier séparé.
	 * */
	public function editEvents() {
		# Génération du fichier XML
		if(empty($this->_aEvents)) {
			if(file_exists(self::DATAFILE)) {
				unlink(self::DATAFILE);
			}
			return;
		}

		# on trie $datas selon la valeur pour la clé self::FIELDNAME
		uasort($this->_aEvents, function($a, $b) {
			$result = strcmp($a[self::FIELDNAME], $b[self::FIELDNAME]);
			if($result != 0) {
				# tri sur dates d'évènements
				return $result;
			}

			# tri sur dates de publication
			return strcmp($a['date'], $b['date']);
		});
		ob_start();
?>
<document>
<?php
		foreach($this->_aEvents as $art_id => $value) {
?>
	<article number="<?= $art_id ?>" date="<?= $value['date'] ?>" active="<?= $value['active'] ?>"><?= $value[self::FIELDNAME] ?></article>
<?php
		}
?>
</document>
<?php
		# On écrit le fichier
		plxUtils::write(self::XML_HEADER . ob_get_clean(), self::DATAFILE);
	}

	/*
	 * Suppression de la valeur du nouveau champ en cas de suppression de l'article.
	 * */
	public function delEvent($artId) {
		if(array_key_exists($artId, $this->_aEvents)) {
			unset($this->_aEvents[$artId]);
			$this->editEvents();
		}
	}

	/**
	 * Affiche le nouveau champ dans le tableau des articles si valeur non nulle
	 * */
	public function AdminIndexFootAppend() {
		if(empty($this->_aEvents)) {
			return;
		}

		$title = addslashes($this->getLang('ARTICLE_EVENT'));
?>
<script> <!-- <?= __CLASS__ ?> plugin -->
	(function() {
		'use strict';
		const links = document.querySelectorAll('#articles-table td:nth-child(4) > a[href^="article.php\?a="]');
		const dateEvents = {
<?php
		foreach($this->_aEvents as $artId=>$values) {
			$dateEvent = plxDate::formatDate(str_replace('-', '', $values[self::FIELDNAME]));
?>
			'<?= $artId ?>' : '<?= $dateEvent ?>',
<?php
		}
?>
		};

		if(links) {
			Array.from(links).forEach(function(el) {
				const id = el.href.replace(/^.*a=(\d+).*/, '$1');
				if(id && id in dateEvents) {
					const tag = document.createElement('em');
					tag.textContent = '(' + dateEvents[id] + ')';
					tag.title = '<?= $title ?>';
					el.insertAdjacentHTML('afterend', '&nbsp;'); // AJoute un espace insécable
					el.parentElement.append(tag);
				}
			});
		}
	})();
</script>
<?php	}

	public function onDelete() {
		if(file_exists(self::DATAFILE)) {
			unlink(self::DATAFILE);
		}
	}

	/*
	 * Récupère la date d'évènement selon l'identifiant de l'article si elle existe.
	 * */
	 public function getEvent($artId) {
		return (is_array($this->_aEvents) and array_key_exists($artId, $this->_aEvents)) ? $this->_aEvents[$artId][self::FIELDNAME] : '';
	}

	/* === Hooks === */

	/*
	 * Récupération des dates d'évènements au démarrage de PluXml.
	 * */
	public function plxMotorConstruct() {
		# Mimics plxMotor::getTags()
		$array = array();
		if(file_exists(self::DATAFILE)) {
			$data = implode('',file(self::DATAFILE));
			$parser = xml_parser_create(PLX_CHARSET);
			xml_parser_set_option($parser,XML_OPTION_CASE_FOLDING,0);
			xml_parser_set_option($parser,XML_OPTION_SKIP_WHITE,0);
			xml_parse_into_struct($parser,$data,$values,$iTags);
			xml_parser_free($parser);
			# On vérifie s'il existe des tags 'article'
			if(isset($iTags['article'])) {
				foreach($iTags['article'] as $idx) {
					$item = $values[$idx];
					$array[ $item['attributes']['number'] ] = array(
						self::FIELDNAME  => isset($item['value']) ? trim($item['value']) : '',
						'date'   => $item['attributes']['date'],
						'active' => $item['attributes']['active'],
					);
				}
			}
		}
		$this->_aEvents = $array;
	}

	/*
	 * On récupère la date d'évènement pour l'édition d'un article via la méthode self::getEvent().
	 * */
	public function plxMotorParseArticle() {
		echo self::BEGIN_CODE;
?>
$art['<?= self::FIELDNAME ?>'] = $this->plxPlugins->aPlugins['<?= __CLASS__ ?>']->getEvent($tmp['artId']);
<?php
		echo self::END_CODE;
	}

	/* === Hooks for back-office === */

	public function AdminArticlePreview() {
		echo self::BEGIN_CODE;
?>
$art['<?= self::FIELDNAME ?>'] = $_POST['<?= self::FIELDNAME ?>'];
<?php
		echo self::END_CODE;
	}

	public function AdminArticlePostData() {
		echo self::BEGIN_CODE;
?>
$kzEvent = $_POST['<?= self::FIELDNAME ?>'];
<?php
		echo self::END_CODE;
	}

	public function AdminArticleParseData() {
		echo self::BEGIN_CODE;
?>
$kzEvent = $result['<?= self::FIELDNAME ?>'];
<?php
		echo self::END_CODE;
	}

	/*
	 * Inialise la date d'évènement pour un nouvel article.
	 * */
	public function AdminArticleInitData() {
		echo self::BEGIN_CODE;
?>
$kzEvent = '';
<?php
		echo self::END_CODE;
	}

	/*
	 * Sauvegarde la date d'évènement via la méthode self::editEvent()
	 * */
	public function plxAdminEditArticleEnd() {
		echo self::BEGIN_CODE;
?>
$this->plxPlugins->aPlugins['<?= __CLASS__ ?>']->editEvent($id, array(
	'<?= self::FIELDNAME ?>' => trim($content['<?= self::FIELDNAME ?>']),
	'date' => $time,
	'active' => intval(!in_array('draft', $content['catId'])),
));
<?php
		echo self::END_CODE;
	}

	/*
	 * Supprime éventuellement le champ évènement si l'article est supprimé.
	 * */
	public function plxAdminDelArticle() {
		echo self::BEGIN_CODE;
?>
if($resDelArt AND $resDelCom) {
	$this->plxPlugins->aPlugins['<?= __CLASS__ ?>']->delEvent($id);
}

# Process must continue for tags
return false;
<?php
		echo self::END_CODE;
	}

	/*
	 * On bufférise la sortie HTML pour positionner le nouveau champ de saisie.
	 * */
	public function AdminArticleContent() {
		ob_start();
		ob_start(); # Against plxPlugins::callHook()
	}

	/*
	 * Ajoute dans la sidebar de l'édition d'un article un champ pour saisir un champ type date.
	 * Le champ est positionné après les dates origniales de PluXml via preg_replace().
	 * */
	public function AdminArticleSidebar() {
		ob_get_clean(); # Against plxPlugins::callHook()
		$content = ob_get_clean();

		ob_start();
		$caption = $this->getLang('ARTICLE_EVENT');
?>
						<label for="id_<?= self::FIELDNAME ?>"><?= $caption ?>&nbsp;:</label>
<?php
		echo '<?php ';
?>
plxUtils::printInput('<?= self::FIELDNAME ?>', $kzEvent, '<?= self::FIELDTYPE ?>', '');
<?php
		echo ' ?>' . PHP_EOL;
?>
					</div>
				</div>
				<div class="grid">
					<div class="col sml-12">
<?php
		$replace = ob_get_clean();
		echo preg_replace('#(<label>' . L_ARTICLE_CATEGORIES . '\&nbsp;:</label>)#', $replace . "$1", $content);
	}

	/*
	 * Affiche la date des évènements dans le tableau des articles via la méthode self::AdminIndexFootAppend())
	 * */
	public function AdminIndexFoot() {
		echo self::BEGIN_CODE;
?>
if(!empty($arts)) {
	$plxAdmin->plxPlugins->aPlugins['<?= __CLASS__ ?>']->AdminIndexFootAppend();
}
<?php
		echo self::END_CODE;
	}

	/* === Hooks for thema === */

	/*
	 * @parameter params type string : 'before', 'after', 'all'
	 * */
	public function kzEvent($params) {
		# Emulation de plxMotor::getArticles('all')
		$motif = false;
		if(!empty($this->_aEvents)) {
			if(!is_string($params)) {
				$params = 'after';
			}
			$today = date('Y-m-d');
			$events = array_filter($this->_aEvents, function($value) use($params, $today) {
				if(empty($value['active'])) {
					return false;
				}
				switch($params) {
					case 'after': return (strcmp($today, $value[self::FIELDNAME]) <= 0); break;
					case 'before': return (strcmp($today, $value[self::FIELDNAME]) > 0); break;
					default: return true;
				}
			});

			if(!empty($events)) {
				$motif = (count($events) > 1) ?'#^(' . implode('|', array_keys($events)) . ')\.#' : '#^' . array_keys($events)[0] . '\.#';
			}
		}

		$plxMotor = plxMotor::getInstance();
		if(!empty($motif) and ($aFiles = $plxMotor->plxGlob_arts->query($motif, 'art'))) {
			# On analyse tous les fichiers
			$artsList = array();
			foreach($aFiles as $v) {
				$art = $plxMotor->parseArticle(PLX_ROOT . $plxMotor->aConf['racine_articles'] . $v);
				if(!empty($art)) {
					$artsList[] = $art;
				}
			}

			# On trie selon les dates d'évènements
			switch($params) {
				case 'after':
				case '':
					# tri croissant
					usort($artsList, function($a, $b) {
						$test = strcmp($a[self::FIELDNAME], $b[self::FIELDNAME]);
						if($test === 0) {
							# 2 évènements le même jour
							return strcmp($a['numero'], $b['numero']);
						}
					    return $test;
					});
					break;
				case 'before':
					# tri décroissant
					usort($artsList, function($a, $b) {
						$test = strcmp($b[self::FIELDNAME], $a[self::FIELDNAME]);
						if($test === 0) {
							# 2 évènements le même jour
							return strcmp($a['numero'], $b['numero']);
						}
					    return $test;
					});
					break;
				default:
					# tri croissant pour le futur et décroissanr pour le passé
					usort($artsList, function($a, $b) use($today) {
						$test = strcmp($a[self::FIELDNAME], $b[self::FIELDNAME]);
						if($test === 0) {
							# 2 évènements le même jour
							return strcmp($a['numero'], $b['numero']);
						}

						$testB = (strcmp($b[self::FIELDNAME], $today) >= 0);
						if(strcmp($a[self::FIELDNAME], $today) >= 0) {
							# évènement du jour ou prochain
							return $testB ? $test : -1;
						} else {
							# évènement passé
							return $testB ? 1 : -$test;
						}
					});
			}

			# On stocke les enregistrements dans un objet plxRecord
			$plxMotor->plxRecord_arts = new plxRecord($artsList);
?>
<!--
<p><?= count($artsList) ?> events</p>
-->
<?php
			return true;
		} else {
			# Aucun évènement à afficher
?>
<p><?php $this->lang('NO_EVENT'); ?></p>
<?php
			return false;
		}
	}

}
